// file      : xsd/cxx/xml/elements.hxx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_XML_ELEMENTS_HXX
#define XSD_CXX_XML_ELEMENTS_HXX

#include <string>

#include <xercesc/util/PlatformUtils.hpp>

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      template <typename C>
      std::basic_string<C>
      prefix (std::basic_string<C> const& n)
      {
        std::size_t i (0);

        while (i < n.length () && n[i] != ':')
          ++i;

        //std::wcerr << "prefix " << n << " "
        //           << std::wstring (n, i == n.length () ? i : 0, i)
        //           << std::endl;

        return std::basic_string<C> (n, i == n.length () ? i : 0, i);
      }

      template <typename C>
      std::basic_string<C>
      uq_name (std::basic_string<C> const& n)
      {
        std::size_t i (0);

        while (i < n.length () && n[i] != ':')
          ++i;

        return std::basic_string<C> (
          n.c_str () + (i == n.length () ? 0 : i + 1));
      }

      //
      //

      inline void
      initialize ()
      {
        xercesc::XMLPlatformUtils::Initialize();
      }

      inline void
      terminate ()
      {
        xercesc::XMLPlatformUtils::Terminate();
      }

      struct auto_initializer
      {
        auto_initializer ()
        {
          initialize ();
        }

        ~auto_initializer ()
        {
          terminate ();
        }
      };
    }
  }
}

#endif  // XSD_CXX_XML_ELEMENTS_HXX
