// file      : xsd/cxx/xml/sax/error-handler.ixx
// author    : Boris Kolpackov <boris@codesynthesis.com>
// copyright : Copyright (c) 2005 Code Synthesis Tools CC
// license   : GNU GPL v2 + exceptions; see accompanying LICENSE file

#ifndef XSD_CXX_XML_SAX_ERROR_HANDLER_IXX
#define XSD_CXX_XML_SAX_ERROR_HANDLER_IXX

#include <iostream>

#include <xsd/cxx/xml/string.hxx>

#include <xercesc/sax/SAXParseException.hpp>

#endif // XSD_CXX_XML_SAX_ERROR_HANDLER_IXX


#if defined(XSD_USE_CHAR) || !defined(XSD_USE_WCHAR)

#ifndef XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_CHAR
#define XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_CHAR

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      namespace sax
      {
        template<>
        inline
        void error_handler<char>::
        handle (xercesc::SAXParseException const& e, severity::value severity)
        {
          XMLCh const* id (e.getPublicId ()
                           ? e.getPublicId ()
                           : e.getSystemId ());

          std::cerr << transcode<char> (id) << ':'
                    << e.getLineNumber () << ':'
                    << e.getColumnNumber () << ": ";

          switch (severity)
          {
          case severity::warning:
            {
              std::cerr << "warning: ";
              break;
            }
          default:
            {
              std::cerr << "error: ";
              break;
            }
          }

          std::cerr << transcode<char> (e.getMessage()) << std::endl;
        }
      }
    }
  }
}

#endif // XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_CHAR
#endif // XSD_USE_CHAR


#if defined(XSD_USE_WCHAR) || !defined(XSD_USE_CHAR)

#ifndef XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_WCHAR
#define XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_WCHAR

namespace xsd
{
  namespace cxx
  {
    namespace xml
    {
      namespace sax
      {
        template<>
        inline
        void error_handler<wchar_t>::
        handle (xercesc::SAXParseException const& e, severity::value severity)
        {
          XMLCh const* id (e.getPublicId ()
                           ? e.getPublicId ()
                           : e.getSystemId ());

          std::wcerr << transcode<wchar_t> (id) << L':'
                     << e.getLineNumber () << L':'
                     << e.getColumnNumber () << L": ";

          switch (severity)
          {
          case severity::warning:
            {
              std::wcerr << L"warning: ";
              break;
            }
          default:
            {
              std::wcerr << L"error: ";
              break;
            }
          }

          std::wcerr << transcode<wchar_t> (e.getMessage()) << std::endl;
        }
      }
    }
  }
}

#endif // XSD_CXX_XML_SAX_ERROR_HANDLER_IXX_WCHAR
#endif // XSD_USE_WCHAR
